// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - LOBBY [CORE]
// Contains core elements for the lobby menu
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#include "ZT2Lobby_Top.h"
#include "ZT2Lobby_OView.h"
#include "ZT2Lobby_Squad.h"
#include "ZT2Lobby_Perks.h"

#define UNBOUND_FADETIME			0.5
#define UNBOUND_HOLDTIME			1.0

enum LobbyModeType
{
	LM_OVERVIEW,					// Match Overview
	LM_PERKS,						// Perks
};

namespace LBCore
{
	using ZTMaterials;
	using LBTop;
	using LBSquad;
	using LBOverview;
	using LBPerks;
	using Scrollbars;
	
	int LobbyMode;								// Which menu is the lobby on?
	
	private int PlayerInput, PlayerInputOld;
	
	// For checking unbound junk
	private int UnboundTick, UnboundGoal;
	
	// ------------------------/
	// Tell the game our lobby menu SHOULD be open
	// Server -> Client
	// ------------------------/
	
	script "EnableLobbyMenu" CLIENTSIDE
	{
		GameState.LobbyOpen = true;
	}
	
	// ------------------------/
	// Get our vertical "center"
	// ------------------------/
	
	int GetLobbyCenter()
	{
		int topY = LBTop::BarY + LOBBTOPM.H;
		int gap = (ResolutionY() * 1.0) - topY;
		
		// Get the top bar Y
		return topY + FixedMul(gap, 0.5);
	}
	
	// ------------------------/
	// Get our horizontal "center"
	// ------------------------/
	
	int GetLobbyCenterX()
	{
		// Get X of squad menu
		int squadX = LBSquad::SquadX;
		return FixedMul(squadX, 0.5);
	}

	// ------------------------/
	// Initialize lobby elements
	// ------------------------/
	
	private void L_Init() 
	{
		Blackness_Draw();
		Blackness_Draw(HID_FADENESS, "MENUWHIT", 0.50);
		
		UnboundGoal = ShaveInt( FixedMul(35.0, UNBOUND_FADETIME + UNBOUND_FADETIME + UNBOUND_HOLDTIME) );
		UnboundTick = 99999;
		
		ChangeCamera(2,1,0);
		DirtyReady = true;
		L_Generate();
		
		LBTop::LT_Init();
		LBSquad::LBS_Init();
		
		LBOverview::LBO_Init();
		LBPerks::LBP_Init();
	}
	
	// ------------------------/
	// Draw trader elements
	// ------------------------/
	
	private void L_Draw() 
	{
		UpdateCursor(true);

		LBTop::LT_Draw();
		LBSquad::LBS_Draw();
		
		LBOverview::LBO_Draw();
		LBPerks::LBP_Draw();
	}
	
	// ------------------------/
	// Tick logic
	// ------------------------/
	
	private void L_Tick()
	{
		L_Unbound_Tick();
		
		PlayerInput = GetPlayerInput(ConsolePlayerNumber(), INPUT_BUTTONS);
		PlayerInputOld = GetPlayerInput(ConsolePlayerNumber(), INPUT_OLDBUTTONS);
		
		// Pressed left click for the first time
		if (ButtonPressed(BT_ATTACK, PlayerInput, PlayerInputOld))
			L_MouseClick();
		else if (ButtonReleased(BT_ATTACK, PlayerInput, PlayerInputOld))
			L_MouseRelease();
			
		Scrollbars::Bar_Tic();
				
		LBTop::LT_Tick();
		LBSquad::LBS_Tick();
		
		LBOverview::LBO_Tick();
		LBPerks::LBP_Tick();
	}
	
	// ------------------------/
	// Left click
	// ------------------------/
	
	private void L_MouseClick()
	{
		LBSquad::LBS_MouseClick();
		LBTop::LT_MouseClick();
		LBPerks::LBP_MouseClick();
	}
	
	// ------------------------/
	// Right click!
	// ------------------------/
	
	private void L_MouseRelease()
	{
		Scrollbars::Bar_Release();
	}
	
	// ------------------------/
	// Cleanup lobby
	// ------------------------/
	
	private void L_Cleanup() 
	{
		Blackness_Cleanup();
		Blackness_Draw(HID_FADENESS, "MENUBLAK", 0.25);
		
		DrawCursor("");
		
		LBTop::LT_Cleanup();
		LBSquad::LBS_Cleanup();
		
		LBOverview::LBO_Cleanup();
		LBPerks::LBP_Cleanup();
	}
	
	// ------------------------/
	// Generate the UI elements
	// Done when resolution changes, etc.
	// ------------------------/
	
	private void L_Generate() {}
	
	// ------------------------/
	// Switch lobby modes!
	// ------------------------/
	
	void L_SwitchModes(int newMode)
	{
		// Cleanup all menus
		LBOverview::LBO_Cleanup();
		LBPerks::LBP_Cleanup();
		
		// Set the new menu
		LobbyMode = newMode;
		
		LBOverview::LBO_Generate(true);
		LBPerks::LBP_Generate(true);
		
		// Dirty the tabs on the top bar so they're re-drawn
		LBTop::LT_DirtyTabs();
	}
	
	//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
	
	// ------------------------/
	// Warn the player they have unbound keys
	// ------------------------/
	
	void L_Unbound_Tick()
	{
		UnboundTick ++;
		
		if (UnboundTick >= UnboundGoal)
		{
			UnboundTick = 0;
			L_Unbound_Show();
		}
	}
	
	// ------------------------/
	// Show the unbound key notification
	// ------------------------/
	
	void L_Unbound_Show()
	{
		Input_CheckBindings();
		
		if (!GameState.HUD.NeedsBinding)
			return;
			
		int ubX = GetLobbyCenterX();
		int ubY = GetLobbyCenter();
		
		str showString = StrParam(s:"\cIYou have unbound ZT2 keys!\n\cKThese are required for gameplay.\n\n", s:GameState.HUD.BindingString);
		
		SetFont("LOBBKEYS");
		HUDMessageUnscaled(ubX, ubY, "a", ALIGN_CENTER, ALIGN_CENTER, HID_LOBBY_KEYBG, UNBOUND_HOLDTIME, UNBOUND_FADETIME, HUDMSG_FADEINOUT, UNBOUND_FADETIME);
		
		SetFont(FNT_UNBOUND);
		HUDMessageUnscaled(ubX, ubY, showString, ALIGN_CENTER, ALIGN_CENTER, HID_LOBBY_KEYTEXT, UNBOUND_HOLDTIME, UNBOUND_FADETIME, HUDMSG_FADEINOUT, UNBOUND_FADETIME);
	}
}