// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - TRADER [BUY MENU]
// The buy menu, on the left
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define BUYMENU_X						16.0
#define BUYMENU_Y						32.0
#define BUYMENU_PADDING					5.0
#define BUYMENU_HORPADDING				72.0
#define BUYMENU_TOPPADDING				90.0
#define BUYMENU_PERKWIDTH				241.0

#define MAX_TRADER_ITEMS				200

// How wide is a price tag?
#define PRICETAG_LENGTH					90.0

// How far apart on the tag should we draw price?
#define PRICETAG_PADDING				10.0

// Padding to use from the left of the buy box
#define BUYPAD_NAME						6.0

// This allows us to share the namespace between Buy and Sell
struct BuyMenuProperties
{
	int ItemsShown;
	
	int Mode;					// Which MODE is this menu designed for?
	
	int HIDBackground;			// Starting HID for background
	int HIDItems;				// Starting HID for items
	int HIDScroll;				// Starting HID for scrollbar
	
	int BuyScrollX;				// X position of scrollbar
	int BuyItemX; 				// Where should we draw the ITEMS?
	int BuyMenuX;				// Where should we draw the BACKGROUND?
	int BuyMenuHeight;			// Actual height of the menu
	
	int ResX;					// Stored X resolution
	int ResY;					// Stored Y resolution
	
	int itemCount;				// How many items CAN we show in the menu?
	int ItemHover;				// Which item are we hovering over?
	
	// These are the items shown ON THE SCREEN
	struct TraderMenuItem ItemList[TRADER_ITEMLIST_CAP];
	
	bool Dirty;					// Things need to be redrawn!
	bool DirtyItems;			// The ITEMS need to be redrawn
	bool DirtyBack;				// Back button needs redrawn
	
	bool LeftAlign;				// This menu is shown on the LEFT of the screen
	
	bool UsesBack;				// This menu uses a BACK button
	bool HoverBack;				// Back button is being hovered over
	
	struct Rect BuyScrollbarR;
	struct ScrollBarT BuyScrollbar;
	struct Rect BackRect;
	
	int FirstIndex;				// First-most index we can use for weps
	int LastBarPos;				// Stores the last Y position of the scrollbar
	
	// For selling, this is our inventory
	// For buying, this is the trader list
	// For perk menu, this is the list of perks
	struct TraderMenuCache BuyItems[TRADER_STORAGE_CAP];
	int BuyItemsSize;
};

namespace TElem_BuyMenu
{
	using ZTMaterials;
	using TElem;
	using Scrollbars;
	
	private int CurrentPerk;			// Which perk are we filtering by?
	private int LastMode;
	private struct BuyMenuProperties Prop_Buy, Prop_Sell, Prop_Perks;
	
	// ------------------------/
	// Initialize core elements
	// ------------------------/
	
	void M_Init()
	{
		// Sync trader mode
		LastMode = TraderMode;
		
		M_InitCall(Prop_Buy, false, MODE_BUY);
		M_InitCall(Prop_Sell, true, MODE_ANY);
		M_InitCall(Prop_Perks, false, MODE_PERK);
	}
	
	private void M_InitCall(struct BuyMenuProperties& Prop, bool isSellMenu, int TheMode)
	{
		Prop.ResX = ResolutionX() * 1.0;
		Prop.ResY = ResolutionY() * 1.0;
		
		Prop.Mode = TheMode;
		
		Prop.LeftAlign = isSellMenu;
		
		Prop.UsesBack = (TheMode == MODE_BUY || TheMode == MODE_UPGRADE);
		
		if (!isSellMenu)
		{
			Prop.HIDBackground = HID_BUYMENU_BG;
			Prop.HIDItems = HID_BUYMENU_ITEM;
			Prop.HIDScroll = HID_BUYMENU_SCROLL;
		}
		else
		{
			Prop.HIDBackground = HID_SELLMENU_BG;
			Prop.HIDItems = HID_SELLMENU_ITEM;
			Prop.HIDScroll = HID_SELLMENU_SCROLL;
		}
		
		M_GenerateCall(Prop);
		M_PopulateCall(Prop);
	}
	
	// ------------------------/
	// Draw core elements
	// ------------------------/
	
	void M_Draw()
	{
		// Our trader mode didn't match!
		// Let's cleanup and start fresh
		
		if (LastMode != TraderMode)
		{
			LastMode = TraderMode;
			
			// EDGE CASE
			// Switching to BUY from SOMETHING ELSE?
			// Let's regenerate, just in case expensive items are now affordable
			if (TraderMode == MODE_BUY)
				M_MoneyChanged();
			
			M_Cleanup();
			M_Populate();
		}
		
		M_DrawCall(Prop_Buy);
		M_DrawCall(Prop_Sell);
		M_DrawCall(Prop_Perks);
	}
	
	private void M_DrawCall(struct BuyMenuProperties& Prop)
	{
		int RealResX = ResolutionX() * 1.0;
		int RealResY = ResolutionY() * 1.0;
		
		// Mode doesn't match
		if (TraderMode != Prop.Mode && Prop.Mode != MODE_ANY)
			return;
		
		// Resolutions are out of sync! Re-populate items
		if (Prop.ResX != RealResX || Prop.ResY != RealResY)
		{
			Prop.ResX = RealResX;
			Prop.ResY = RealResY;
			M_CleanupCall(Prop);
			M_PopulateCall(Prop);
		}
		
		M_TickCall(Prop);
		
		bool barWasDirty = ScrollbarDirty(Prop.BuyScrollbar);
		
		if (!Prop.Dirty && !barWasDirty && !Prop.DirtyItems && !Prop.DirtyBack)
			return;
			
		if (barWasDirty)
		{
			barWasDirty = false;
			Bar_Draw(Prop.BuyScrollbar);
		}
			
		// Draw core things
		if (Prop.Dirty)
		{
			Prop.Dirty = false;
				
			// Decide our height
			// Our resolution can change!
			
			M_SetXPosition(Prop);
			
			// Background
			HUDMessageRectangle(Prop.BuyMenuX, BUYMENU_Y, Prop.BuyMenuHeight, Prop.HIDBackground, TRADBG1, TRADBG2, TRADBG2);
		}
		
		// Draw the items
		if (Prop.DirtyItems)
		{
			Prop.DirtyItems = false;
			int listHID = Prop.HIDItems;
			
			// Loop through all items...
			for (int i=0; i<Prop.ItemsShown; i++)
			{
				// Set listHID according to what it returns
				listHID = M_DrawBuyItem(Prop, i, listHID);
			}
		}
		
		// Back button
		if (Prop.DirtyBack && Prop.UsesBack)
		{
			Prop.DirtyBack = false;
			HUDMessageMaterial(Prop.HoverBack ? TRADBAK2 : TRADBAK1, Prop.BackRect, HID_BUYMENU_BACK);
		}
	}
	
	// ------------------------/
	// Populate the list with items!
	// ------------------------/
	
	void M_Populate(bool ifShown = false) 
	{
		M_PopulateCall(Prop_Buy, ifShown);
		M_PopulateCall(Prop_Sell, ifShown);
		M_PopulateCall(Prop_Perks, ifShown);
	}
	
	private void M_PopulateCall(struct BuyMenuProperties& Prop, bool ifShown = false)
	{
		// If it needs to be shown, then check if it is
		if (ifShown && (TraderMode != Prop.Mode && Prop.Mode != MODE_ANY))
			return;
			
		Prop.Dirty = true;
		Prop.DirtyItems = true;
		Prop.DirtyBack = true;

		Prop.ItemsShown = 0;
		Prop.ItemHover = -1;
		
		int startYoffset = BUYMENU_TOPPADDING;
		int startY = BUYMENU_Y + startYoffset;
		
		M_SetXPosition(Prop);
		
		// How many items CAN we show?
		int itemHeight = TRADBUY1.H + BUYMENU_PADDING;
		
		Prop.BuyMenuHeight = DecideSideHeight();
		
		// We don't calculate item count based on BACKGROUND HEIGHT, but item rect
		int barHeight = (ResolutionY() * 1.0) - startY;
		int itemArea = barHeight;
		
		// Uses back? Subtract the back button height
		if (Prop.UsesBack)
			itemArea -= TRADBAK1.H;
		
		Prop.itemCount = ShaveInt( FixedDiv( itemArea, itemHeight ) ) - 1;
		
		M_UpdateItemRange(Prop);
		
		// Set the scrollbar position
		SetRect(Prop.BuyScrollbarR, Prop.BuyScrollX, startY, TRADSBR1.W, barHeight);
		SetScrollbar(Prop.BuyScrollbar, Prop.BuyScrollbarR, Prop.HIDScroll);
		
		// Is it even needed?
		if (Prop.BuyItemsSize > Prop.itemCount)
			Bar_Enable(Prop.BuyScrollbar);
		else
			Bar_Disable(Prop.BuyScrollbar);
		
		// Does it use a BACK button?
		// If so, determine where it needs to go
		if (Prop.UsesBack)
		{
			int BackX = ((Prop.BuyMenuX + TRADBG1.W) - TRADBAK1.W) - TRADSBR1.W;
			int BackY = (ResolutionY() * 1.0) - TRADBAK1.H;
			SetRect(Prop.BackRect, BackX, BackY, TRADBAK1.W, TRADBAK1.H);
		}
	}
	
	// ------------------------/
	// Behavior-based functionality
	// ------------------------/
	
	void M_Tick()
	{
		M_TickCall(Prop_Buy);
		M_TickCall(Prop_Sell);
		M_TickCall(Prop_Perks);
	}
	
	private void M_TickCall(struct BuyMenuProperties& Prop)
	{
		int LastHover = Prop.ItemHover;
		Prop.ItemHover = -1;
		
		int MX = GetMouseX();
		int MY = GetMouseY();
		
		// Left and right boundaries of our menu (Don't count pricetags)
		int xMin;
		int itWidth = TRADBUY1.W - PRICETAG_LENGTH;
		
		// Perk? Use perk image width
		if (Prop.Mode == MODE_PERK)
			itWidth = BUYMENU_PERKWIDTH;
		
		// Sell
		if (Prop.LeftAlign)
			xMin = Prop.BuyItemX;
			
		// Buy
		else
			xMin = Prop.BuyItemX - itWidth;
			
		int xMax = xMin + itWidth;
		
		// Roughly inside of the menu, check for hovers
		if (MX >= xMin && MX <= xMax && !ScrollBars::barHeld)
		{
			for (int i=0; i<Prop.ItemsShown; i++)
			{
				if (MY >= Prop.ItemList[i].Bounds.Y && MY <= Prop.ItemList[i].Bounds.Y+Prop.ItemList[i].Bounds.H && Prop.ItemList[i].Visible)
				{
					Prop.ItemHover = i;
					break;
				}
			}
		}
		else
			Prop.ItemHover = -1;
			
		// DIRTY
		if (LastHover != Prop.ItemHover)
		{
			Prop.DirtyItems = true;
			
			if (Prop.ItemHover >= 0)
			{
				if (Prop.Mode == MODE_PERK)
					LocalAmbientSound("trader/perkover", 127);
				else
					LocalAmbientSound("trader/wepover", 127);
			}
			
			// Update the tooltip with what we hovered over
			if (Prop.ItemHover == -1)
				TElem::M_SetTooltip("Blahblah", "Description", " ", false);
			else
			{
				int realInd = Prop.ItemList[Prop.ItemHover].index;
				
				str tTitle = Prop.BuyItems[realInd].Name;
				str tDesc = "This is a gun, this description is a single line of text in a string variable and is being automatically wrapped by BCS magic";
				str tPic = "BUYPIC00";
				
				TElem::M_SetTooltip(tTitle, tDesc, tPic, true);
			}
		}
			
		// Moved the bar?
		if (Prop.LastBarPos != Prop.BuyScrollbar.BarY)
		{
			Prop.LastBarPos = Prop.BuyScrollbar.BarY;
			
			M_UpdateItemRange(Prop);
			Prop.DirtyItems = true;
		}
		
		// Back button needs hover check
		if (Prop.UsesBack)
		{
			bool HB = (Mousing(Prop.BackRect) && !ScrollBars::barHeld);
			
			if (Prop.HoverBack != HB)
			{
				Prop.HoverBack = HB;
				Prop.DirtyBack = true;
			}
		}
	}
	
	// ------------------------/
	// Mouse clicked
	// ------------------------/
	
	void M_MouseClick()
	{
		M_MouseClickCall(Prop_Buy);
		M_MouseClickCall(Prop_Sell);
		M_MouseClickCall(Prop_Perks);
	}
	
	private void M_MouseClickCall(struct BuyMenuProperties& Prop)
	{
		// Mode doesn't match
		if (TraderMode != Prop.Mode && Prop.Mode != MODE_ANY)
			return;
			
		// Scrollbar functionality if we're hovering over it
		if (ScrollBars::Bar_Hovered(Prop.BuyScrollbar))
		{
			Scrollbars::Bar_AttemptSet(Prop.BuyScrollbar);
			return;
		}
		
		// Hovering over back button
		if (Prop.HoverBack && Prop.UsesBack)
		{
			LocalAmbientSound("trader/back", 127);
			
			// Go back!
			if (Prop.Mode == MODE_UPGRADE)
				M_SetTraderMode(MODE_BUY);
			else
				M_SetTraderMode(MODE_PERK);
				
			return;
		}
		
		// Hovering over an item?
		if (Prop.ItemHover >= 0)
		{
			// Index in BuyItems table
			int BIIndex = Prop.ItemList[Prop.ItemHover].Index;
			
			// REAL index in WeaponList table
			int RealIndex = Prop.BuyItems[BIIndex].Index;
			
			// Item!
			if (Prop.Mode == MODE_BUY)
			{
				LocalAmbientSound("trader/wepclick", 127);
					
				M_BuyWeapon(RealIndex);
				return;
			}
			
			// Sell!
			if (Prop.LeftAlign)
			{
				M_SellWeapon(RealIndex);
				return;
			}
			
			// Perks!
			if (!Prop.LeftAlign && Prop.Mode == MODE_PERK)
			{
				CurrentPerk = RealIndex;
				
				LocalAmbientSound("trader/perkclick", 127);
				
				// Wait a minute!
				
				// Before we set buy mode, let's re-populate our items!
				// This will filter them according to the perk we have selected
				
				M_MoneyChanged();
				
				M_SetTraderMode(MODE_BUY);
				return;
			}

			SetFont(FNT_PLAIN);
			PrintBold(s:"HOVERING OVER ITEM ", d:Prop.ItemList[Prop.ItemHover].Index);
		}
	}
	
	// ------------------------/
	// Mouse released
	// ------------------------/
	
	void M_MouseRelease() {}
	
	// ------------------------/
	// Generate our list of items
	// Each menu has different items!
	// ------------------------/
	
	void M_Generate(bool ifShown = false)
	{
		M_GenerateCall(Prop_Buy, ifShown);
		M_GenerateCall(Prop_Sell, ifShown);
		M_GenerateCall(Prop_Perks, ifShown);
	}
	
	void M_GenerateCall(struct BuyMenuProperties& Prop, bool ifShown = false)
	{
		int i;
		
		// If it's required to be shown, then make sure it is
		if (ifShown && (TraderMode != Prop.Mode && Prop.Mode != MODE_ANY))
			return;
		
		Prop.BuyItemsSize = 0;
		
		//-- PERK SELECT --
		if (Prop.Mode == MODE_PERK)
		{
			for (i=0; i<PerkListSize; i++)
			{
				if (!PerkList[i].Hidden)
				{
					Prop.BuyItems[Prop.BuyItemsSize].Name = PerkList[i].DisplayName;
					Prop.BuyItems[Prop.BuyItemsSize].Index = i;
					Prop.BuyItemsSize ++;
				}
			}
			
			return;
		}
		
		//-- BUY MENU
		if (Prop.Mode == MODE_BUY)
		{
			// Loop through ALL available weapons
			for (i=0; i<WeaponListSize; i++)
			{
				// Do we already have it? Don't add it
				if (CheckInventory(WeaponList[i].Class))
					continue;
					
				// Undroppable, don't buy
				if (WeaponList[i].Undroppable)
					continue;
					
				// Check if the wep is perked
				if (!WeaponIsPerked(i, CurrentPerk))
					continue;
			
				Prop.BuyItems[Prop.BuyItemsSize].Index = i;
				Prop.BuyItems[Prop.BuyItemsSize].Name = WeaponList[i].DisplayName;
				Prop.BuyItems[Prop.BuyItemsSize].Price = GetBuyPrice(i);
				
				// Can we buy it?
				Prop.BuyItems[Prop.BuyItemsSize].Denied = (CheckInventory("Money") < WeaponList[i].Price);
				
				Prop.BuyItemsSize ++;
			}
			return;
		}
		
		//-- SELL MENU
		if (Prop.LeftAlign)
		{
			// Loop through ALL weapons
			for (i=0; i<WeaponListSize; i++)
			{
				// Do we have it?
				if (CheckInventory(WeaponList[i].Class))
				{
					int ind = Prop.BuyItemsSize;
					
					Prop.BuyItems[Prop.BuyItemsSize].Index = i;
					Prop.BuyItems[ind].Name = WeaponList[i].DisplayName;
					Prop.BuyItems[ind].Price = GetSellPrice(i);
					
					// Undroppable? Can't do anything
					Prop.BuyItems[ind].Denied = WeaponList[i].Undroppable;
					
					Prop.BuyItemsSize ++;
				}
			}
			
			return;
		}
	}
	
	// ------------------------/
	// Update based on scrollbar, etc.
	// ------------------------/
	
	private void M_UpdateItemRange(struct BuyMenuProperties& Prop) 
	{
		// Subtract our POSSIBLE list size from TOTAL list size
		// This ensures that we can show all items on-screen (hard to explain)
		
		int ClampedSize = Prop.BuyItemsSize - Prop.itemCount;

		Prop.FirstIndex = ShaveInt( FixedMul(ClampedSize * 1.0, Prop.BuyScrollbar.Pct) );
		
		Prop.ItemsShown = 0; 
		
		int itemHeight = TRADBUY1.H + BUYMENU_PADDING;
		
		int startYoffset = BUYMENU_TOPPADDING;
		int startY = BUYMENU_Y + startYoffset;
		
		int NewX = Prop.BuyItemX - ( Prop.LeftAlign ? 0.0 : TRADBUY1.W );
		
		int counter = 0;
		for (int i=Prop.FirstIndex; i<Prop.FirstIndex+Prop.itemCount; i++)
		{
			Prop.ItemList[counter].Visible = (i < Prop.BuyItemsSize);
			
			// This points to the index in the BUYITEMS TABLE, NOT the actual weapon!
			Prop.ItemList[counter].Index = i;
				
			SetRect(Prop.ItemList[counter].Bounds, NewX, startY, TRADBUY1.W, TRADBUY1.H);
			
			counter ++;
			startY += itemHeight;
			
			Prop.ItemsShown ++;
		}
	}
	
	// ------------------------/
	// Cleanup the list of items
	// ------------------------/
	
	void M_Cleanup()
	{
		M_CleanupCall(Prop_Buy);
		M_CleanupCall(Prop_Sell);
		M_CleanupCall(Prop_Perks);
	}
	
	private void M_CleanupCall(struct BuyMenuProperties& Prop)
	{
		// Cleanup the scrollbar
		Bar_Cleanup(Prop.BuyScrollbar);
		
		ClearHUDMessageRange(Prop.HIDBackground, 50);
		
		// - Background
		// - Text
		// - Price
		
		int PER_ITEM_COUNT = 3;
		ClearHUDMessageRange(Prop.HIDItems, (Prop.itemCount + 3) * PER_ITEM_COUNT);
		
		// This is mostly for drawing the next time around
		Prop.Dirty = true;
		Prop.DirtyItems = true;
		Prop.DirtyBack = true;
		
		ClearHUDMessage(HID_BUYMENU_BACK);
	}
	
	// ------------------------/
	// Set X position for a menu
	// ------------------------/
	
	private void M_SetXPosition(struct BuyMenuProperties& Prop)
	{
		if (Prop.LeftAlign)
		{
			// Prop.BuyMenuX = BUYMENU_X + TRADSBR1.W;
			Prop.BuyMenuX = BUYMENU_X;
			Prop.BuyItemX = Prop.BuyMenuX + TRADSBR1.W;
			// Prop.BuyScrollX = BUYMENU_X;
			Prop.BuyScrollX = BUYMENU_X;
		}
		else
		{
			Prop.BuyMenuX = ((ResolutionX() * 1.0) - BUYMENU_X) - TRADBG1.W;
			Prop.BuyItemX = (Prop.BuyMenuX + TRADBG1.W) - TRADSBR1.W;
			// Prop.BuyScrollX = Prop.BuyMenuX+TRADBG1.W;
			Prop.BuyScrollX = ((ResolutionX() * 1.0) - BUYMENU_X) - TRADSBR1.W;
		}
	}
	
	// ------------------------/
	// Draw a list iten for a buy menu
	// Returns the NEXT id it should use
	// ------------------------/
	
	private int M_DrawBuyItem(struct BuyMenuProperties& Prop, int i, int listHID)
	{
		if (!Prop.ItemList[i].Visible)
		{
			ClearHUDMessage(listHID);			// BG
			ClearHUDMessage(listHID-1);			// Name
			ClearHUDMessage(listHID-2);			// Price
			listHID -= 3;
			
			return listHID;
		}
		
		int RealIndex = Prop.ItemList[i].Index;
		
		// Takes crash? For buy-items, we can buy it
		// For sell-items, we can sell it
		bool takesCash = !Prop.BuyItems[i].Denied;
		struct Picture& BarPic = TRADBUY8;
		
		// Which image should we use?
		if (Prop.LeftAlign)
		{
			// Sellable
			if (takesCash)
				BarPic = (Prop.ItemHover == i) ? TRADSLI2 : TRADSLI1;
			else
				BarPic = (Prop.ItemHover == i) ? TRADSLI3 : TRADSLI3;
		}
		else
		{
			// Buyable
			if (takesCash)
				BarPic = (Prop.ItemHover == i) ? TRADBUY2 : TRADBUY1;
			else
				BarPic = (Prop.ItemHover == i) ? TRADBUY4 : TRADBUY3;
		}
		
		// Perk overrides it
		if (Prop.Mode == MODE_PERK)
			BarPic = (Prop.ItemHover == i) ? TRADPRK2 : TRADPRK1;
		
		HUDMessageMaterial(BarPic, Prop.ItemList[i].Bounds, listHID);
		listHID --;
		
		// Text!
		str itemName = Prop.BuyItems[RealIndex].Name;
		
		int textX = BUYPAD_NAME;
		
		if (!Prop.LeftAlign)
			textX += PRICETAG_LENGTH;
		
		SetFont(FNT_TRADERNAME);
		HUDMessageUnscaled(Prop.ItemList[i].Bounds.X + textX, Prop.ItemList[i].Bounds.Y + 16.0, itemName, ALIGN_LEFT, ALIGN_CENTER, listHID);
		listHID --;
		
		// Should we draw price text?
		str priceText;
		bool shouldDrawPrice = (Prop.LeftAlign && takesCash) || !Prop.LeftAlign;
		
		// Perks have no price
		if (Prop.Mode == MODE_PERK)
			shouldDrawPrice = false;
		
		if (!shouldDrawPrice)
			priceText = "";
		else
			priceText = StrParam(s:"$", d:Prop.BuyItems[ Prop.ItemList[i].Index ].Price);
			
		// WHERE should we draw it?
		// First, get the length of the "base" bar
		int baseBarLength = TRADBUY5.W;
		
		// Now, determine the X position and alignment of the price
		int tagX, tagAlign;
		
		// Left menus draw on the LEFT of the tab
		if (Prop.LeftAlign)	
		{
			tagX = baseBarLength + PRICETAG_PADDING;
			tagAlign = ALIGN_LEFT;
		}
			
		// Right menus draw on the RIGHT of the tab
		else
		{
			tagX = PRICETAG_LENGTH - PRICETAG_PADDING;
			tagAlign = ALIGN_RIGHT;
		}
			
		SetFont(FNT_TRADERPRICE);
		HUDMessageUnscaled(Prop.ItemList[i].Bounds.X + tagX, Prop.ItemList[i].Bounds.Y + 16.0, priceText, tagAlign, ALIGN_CENTER, listHID);
		listHID --;
		
		return listHID;
	}
	
	// ------------------------/
	// The amount of money we have has changed
	// ------------------------/
	void M_MoneyChanged()
	{
		// Sell items don't need to be generated
		// They only become "denied" based on their initial state
		
		M_GenerateCall(Prop_Buy, true);
		
		// Re-sync the list items
		
		M_UpdateItemRange(Prop_Buy);
		
		// Dirty it up
		
		Prop_Buy.DirtyItems = true;
	}
	
	// ------------------------/
	// Refresh the sell list
	// ------------------------/
	
	void M_RefreshSellList()
	{
		// Regenerate the items
		
		M_GenerateCall(Prop_Sell, true);
		
		// Re-sync the list items
		
		M_UpdateItemRange(Prop_Sell);
		
		// Dirty it up
		
		Prop_Sell.DirtyItems = true;
	}
}