// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - TRADER [ELEMENTS]
// Core trader elements! This is the MAIN TRADER MENU
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#include "ZT2T_BuyMenu.h"
#include "ZT2T_Scrollbar.h"
#include "ZT2T_Tooltip.h"

// Space underneath the EXIT button
#define TRADER_BOTTOMPAD_CENTER 			40.0;

// Space above the EXIT button
#define TRADER_CENTERPAD_CENTER				8.0;

// Amount of tics for "mode swap"
#define TRADER_MODESWAP_TIME				5

// Modes for the trader, mostly for the right-hand menu
enum TraderMenuMode
{
	MODE_PERK,					// Select a perk category!
	MODE_BUY,					// Purchase an item!
	MODE_UPGRADE,				// Upgrade menu
	MODE_ANY,					// Any mode will work
};

// Actually SHOWN in the menu
struct TraderMenuItem
{
	int Index;
	bool Visible;
	struct Rect Bounds;
};

// The TOTAL list of items we can sell
// ListItems refers to this
struct TraderMenuCache
{
	int Index;			// The REAL item this refers to
	str Name;			// Name to show on it
	int Price;			// Price for the item
	bool Denied;		// Can this item be bought, sold, etc.?
};

// Various info about trader-related replication
struct TraderReplicationStruct
{
	bool Sending;				// Waiting on a replication response
	int LastType;				// Last TYPE of request we sent
	int LastArg;				// Last arg sent
	str SuccessString;			// String to show if it was successful
};

namespace TElem
{
	using TElem_BuyMenu;
	using ZTMaterials;
	using Tooltips;
	
	private int PlayerInput, PlayerInputOld;
	
	// The tooltip for the entire trader menu
	private struct ToolTipT TraderTooltip;
	
	// Rectangles
	private struct Rect AmmoRect, ArmorRect, ExitRect, MoneyRect;
	
	// Hovering over certain things?
	private bool HoverAmmo, HoverArmor, HoverExit;
	
	// These elements are dirty and need redrawing
	private bool DirtyAmmo, DirtyArmor, DirtyExit;
	private bool DirtyMoney;
	
	// Last known money amount
	private int LastMoney;
	
	// Last known resolutions
	private int ResX, ResY;
	
	// Replication
	private struct TraderReplicationStruct TraderRep;
	
	// Which mode are we in?
	int TraderMode;
	
	// When we swap modes, a timer starts
	private int ModeSwapTimer;
	
	// ------------------------/
	// Open the actual trader menu!
	// SERVER
	// ------------------------/
	
	script SCRIPT_TRADERMENU { OpenTraderMenu( (CheckInventory("TraderMenuOpen") <= 0) ); }
	
	void OpenTraderMenu(bool show = true)
	{
		// Prevent duplicate events
		if ((show && CheckInventory("TraderMenuOpen")) || (!show && !CheckInventory("TraderMenuOpen")))
			return;
			
		if (show)
		{
			InvEnsure("HideHUD", 1);
			GiveInventory("TraderMenuOpen", 1);
			FreezeMe(true);
		}
		else
		{
			InvEnsure("HideHUD", 0);
			TakeInventory("TraderMenuOpen", 1);
			FreezeMe(false);
		}
		
		ACS_NamedExecuteAlways("ShowTraderMenu", 0, show ? 1 : 0);
	}
	
	// ------------------------/
	// Physically SHOW the trader menu!
	// CLIENT
	// ------------------------/
	
	script "ShowTraderMenu" (int show) CLIENTSIDE
	{
		if (WrongClient())
			terminate;
			
		if (show)
			M_Init();
	}
	
	script "CloseTraderMenu" CLIENTSIDE
	{
		if (WrongClient())
			terminate;
	}
	
	// ------------------------/
	// Script responsible for drawing THE ENTIRE MENU
	// This loops
	// ------------------------/
	
	script "DrawTraderMenu" CLIENTSIDE
	{
		GameState.HUD.TraderIsOpen = true;
		
		while (CheckInventory("TraderMenuOpen"))
		{
			// Update the cursor FIRST
			// We need to do this so hovered items, etc. can be drawn
			// in the same tic
			
			UpdateCursor(true);
			
			M_Draw();
			delay(1);
		}

		GameState.HUD.TraderIsOpen = false;
		M_Cleanup();
	}
	
	// ------------------------/
	// Initialize core elements
	// ------------------------/
	
	private void M_Init()
	{
		Blackness_Draw();
		Blackness_Draw(HID_FADENESS, "MENUBLAK", 0.5);
		
		LocalAmbientSound("trader/hello", 127);
		
		TraderMode = MODE_PERK;
		
		ModeSwapTimer = 0;
		
		TElem_BuyMenu::M_Init();
		
		// Setup the trader tooltip
		TT_Set(TraderTooltip, " ", " ", HID_TRADER_TOOLTIP);
		TT_SetTheme(TraderTooltip, INFOBOX1, INFOBOX2, INFOBOX3);
		TT_Show(TraderTooltip, false);
		
		// Generate elements for this menu
		M_Generate();
		
		// Now actually draw everything
		ACS_NamedExecute("DrawTraderMenu", 0);
	}
	
	// ------------------------/
	// Draw core elements
	// ------------------------/
	
	private void M_Draw()
	{
		M_Tick();
		TElem_BuyMenu::M_Draw();
		
		// Draw tooltips last
		TT_Draw(TraderTooltip);
		
		// Draw center elements
		M_CenterDraw();
	}
	
	// ------------------------/
	// Decide height for our buy and sell menus
	// ------------------------/
	
	int DecideSideHeight()
	{
		return (ResolutionY() * 1.0) - 32.0;
	}
	
	// ------------------------/
	// Behavior functionality
	// ------------------------/
	
	private void M_Tick()
	{
		if (ModeSwapTimer > 0)
			ModeSwapTimer --;
			
		PlayerInput = GetPlayerInput(-1, INPUT_BUTTONS);
		PlayerInputOld = GetPlayerInput(-1, INPUT_OLDBUTTONS);
		
		// Pressed left click for the first time
		if (ButtonPressed(BT_ATTACK, PlayerInput, PlayerInputOld))
			M_MouseClick();
		
		// Released it
		else if (ButtonReleased(BT_ATTACK, PlayerInput, PlayerInputOld))
			M_MouseRelease();
			
		Tooltips::TT_Tick(TraderTooltip);
		Scrollbars::Bar_Tic();
		
		// Res has changed, re-generate elements
		if (ResX != ResolutionX() || ResY != ResolutionY())
			M_Generate();
			
		// Tick logic for center elements
		M_CenterTick();
	}
	
	// ------------------------/
	// Left click!
	// ------------------------/
	
	private void M_MouseClick()
	{
		// Exit
		if (HoverExit)
		{
			M_ExitTraderMenu();
			return;
		}
		
		TElem_BuyMenu::M_MouseClick();
	}
	
	// ------------------------/
	// Right click!
	// ------------------------/
	
	private void M_MouseRelease()
	{
		TElem_BuyMenu::M_MouseRelease();
		Scrollbars::Bar_Release();
	}
	
	// ------------------------/
	// Accessor to our private variable
	// ------------------------/
	
	void M_SetTooltip(str Title, str Desc, str Picture, bool show = true)
	{
		if (!show)
			TT_Show(TraderTooltip, false);
		else
		{
			TT_Show(TraderTooltip, show);
			if (title != "" || Desc != "")
				TT_Set(TraderTooltip, Title, Desc, -1, Picture);
		}
	}
	
	// ------------------------/
	// Cleanup the menu!
	// ------------------------/
	
	private void M_Cleanup()
	{
		Blackness_Cleanup();
		
		Blackness_Draw(HID_FADENESS, "MENUBLAK", 0.25);
		
		LocalAmbientSound("trader/exit", 127);
		
		TElem_BuyMenu::M_Cleanup();
		M_CenterCleanup();
		
		// Cleanup the cursor
		DrawCursor("");
		
		// Cleanup tooltip
		TT_Show(TraderTooltip, false);
		TT_Draw(TraderTooltip);
	}
	
	// ------------------------/
	// Generate elements
	// Done when initting and when res changes
	// ------------------------/
	
	private void M_Generate()
	{
		// Sync res
		ResX = ResolutionX();
		ResY = ResolutionY();
		
		// Senc money
		LastMoney = CheckInventory("Money");
		
		// Generate center elements
		M_CenterGenerate();
	}
	
	// ------------------------/
	// Center elements (exit, etc.)
	// ------------------------/
	
	private void M_CenterGenerate()
	{
		// Clean 'em up first
		M_CenterCleanup();
		
		// All are dirty, redraw them
		DirtyAmmo = true;
		DirtyArmor = true;
		DirtyExit = true;
		DirtyMoney = true;
		
		// Figure out the coords for the exit box
		int ExitX = (CenterX()*1.0) - FixedMul(TRADEXT1.W, 0.5);
		int ExitY = ((ResY*1.0) - TRADER_BOTTOMPAD_CENTER) - TRADEXT1.H;
		SetRect(ExitRect, ExitX, ExitY, TRADEXT1.W, TRADEXT1.H);
		
		int ArmorY = (ExitY - TRADER_CENTERPAD_CENTER) - TRADARM1.H;
		int ArmorX = ExitX;
		int AmmoX = (ExitX + TRADEXT1.W) - TRADARM1.W;
		SetRect(AmmoRect, AmmoX, ArmorY, TRADAMO1.W, TRADAMO1.H);
		SetRect(ArmorRect, ArmorX, ArmorY, TRADARM1.W, TRADARM2.H);
		
		// Where to draw money?
		int MoneyX = (CenterX()*1.0) - FixedMul(TRADMONY.W, 0.5);
		int MoneyY = 128.0;
		SetRect(MoneyRect, MoneyX, MoneyY, TRADMONY.W, TRADMONY.H);
	}
	
	// ------------------------/
	// Cleanup center elements
	// ------------------------/
	
	private void M_CenterCleanup() 
	{
		ClearHUDMessage(HID_TRADER_EXIT);
		ClearHUDMessage(HID_TRADER_AMMO);
		ClearHUDMessage(HID_TRADER_ARMOR);
		ClearHUDMessage(HID_TRADER_MONEYBG);
		ClearHUDMessage(HID_TRADER_MONEY);
		ClearHUDMessage(HID_TRADER_EXITTEXT);
	}
	
	// ------------------------/
	// Draw center elements
	// ------------------------/
	
	private void M_CenterDraw()
	{
		// Exit button
		if (DirtyExit)
		{
			DirtyExit = false;
			HUDMessageMaterial(HoverExit ? TRADEXT2 : TRADEXT1, ExitRect, HID_TRADER_EXIT);
			
			int xX = ExitRect.X + FixedMul(ExitRect.W, 0.5);
			int xY = (ExitRect.Y + FixedMul(ExitRect.H, 0.5)) - 5.0;
			
			SetFont(FNT_EXITTEXT);
			HUDMessageUnscaled(xX, xY, "EXIT TRADER", ALIGN_CENTER, ALIGN_CENTER, HID_TRADER_EXITTEXT);
		}
		
		// Ammo button
		if (DirtyAmmo)
		{
			DirtyAmmo = false;
			HUDMessageMaterial(HoverAmmo ? TRADAMO2 : TRADAMO1, AmmoRect, HID_TRADER_AMMO);
		}
		
		// Armor button
		if (DirtyArmor)
		{
			DirtyArmor = false;
			HUDMessageMaterial(HoverArmor ? TRADARM2 : TRADARM1, ArmorRect, HID_TRADER_ARMOR);
		}
		
		// Money!
		if (DirtyMoney)
		{
			DirtyMoney = false;
			HUDMessageMaterial(TRADMONY, MoneyRect, HID_TRADER_MONEYBG);
			
			int txtX = MoneyRect.X + FixedMul(MoneyRect.W, 0.5);
			int txtY = MoneyRect.Y + FixedMul(MoneyRect.H, 0.5);
			
			SetFont("TFNDESC");
			HUDMessageUnscaled(txtX, txtY, StrParam(s:"$", d:LastMoney), ALIGN_CENTER, ALIGN_CENTER, HID_TRADER_MONEY);
		}
	}
	
	// ------------------------/
	// Tick logic for centers
	// ------------------------/
	
	private void M_CenterTick()
	{
		bool NewHover;
		
		// -- HOVER LOGIC: EXIT
		NewHover = Mousing(ExitRect);
		if (HoverExit != NewHover)
		{
			HoverExit = NewHover;
			DirtyExit = true;
			
			if (HoverExit)
				LocalAmbientSound("trader/wepover", 127);
		}
		
		// -- HOVER LOGIC: AMMO
		NewHover = Mousing(AmmoRect);
		if (HoverAmmo != NewHover)
		{
			HoverAmmo = NewHover;
			DirtyAmmo = true;
			
			if (HoverAmmo)
				LocalAmbientSound("trader/wepover", 127);
		}
		
		// -- HOVER LOGIC: ARMOR
		NewHover = Mousing(ArmorRect);
		if (HoverArmor != NewHover)
		{
			HoverArmor = NewHover;
			DirtyArmor = true;
			
			if (HoverArmor)
				LocalAmbientSound("trader/wepover", 127);
		}
		
		// -- MONEY IS OUT OF DATE
		int ourMoney = CheckInventory("Money");
		if (LastMoney != ourMoney)
		{
			LastMoney = ourMoney;
			DirtyMoney = true;
			M_MoneyChanged();
		}
	}
	
	// ------------------------/
	// Set the trader mode
	// ------------------------/
	
	void M_SetTraderMode(int nMode)
	{
		// We can only change modes so quickly
		if (M_InModeSwap())
			return;
			
		M_SetTooltip("", "", "", false);
			
		ModeSwapTimer = TRADER_MODESWAP_TIME;
		TraderMode = nMode;
	}
	
	// ------------------------/
	// Are we in modeswap?
	// ------------------------/
	
	bool M_InModeSwap()
	{
		return (ModeSwapTimer > 0);
	}
	
	// ------------------------/
	// The amount of money we have has changed
	// Buy items, etc. need to be updated
	// ------------------------/
	
	private void M_MoneyChanged()
	{
		TElem_BuyMenu::M_MoneyChanged();
	}
	
	//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
	
	// ------------------------/
	// Wait for a server response
	// CLIENT
	// ------------------------/
	script "Rep_TraderWait"
	{
		// Wait a bit before re-sending
		delay(REPLICATION_INTERVAL);
		
		// Time out
		if (TraderRep.Sending)
		{
			M_StartTraderRequest(TraderRep.LastType, TraderRep.SuccessString, TraderRep.LastArg);
		}
	}
	
	// ------------------------/
	// Got a server response!
	// SERVER -> CLIENT
	// ------------------------/
	script "Rep_TraderResponse" (int responseType) CLIENTSIDE
	{
		bool res;
		
		if (WrongClient())
			terminate;
			
		TraderRep.Sending = false;
		ACS_NamedTerminate("Rep_TraderWait", 0);
		
		switch (TraderRep.LastType)
		{
			// Buy a weapon
			case TREQ_BUYWEAPON:
				res = TF_BuyWeaponResponse(responseType, TraderRep.LastArg, TraderRep.SuccessString);
				
				// All good? Update sell list
				if (res)
					M_RefreshSellList();
					
				// Buy menu should auto-refresh because our money changed
			break;
			
			// Sell a weapon
			case TREQ_SELLWEAPON:
				res = TF_SellWeaponResponse(responseType, TraderRep.LastArg, TraderRep.SuccessString);
				
				// All good? Update sell list
				if (res)
					M_RefreshSellList();
					
				// Buy menu should auto-refresh because our money changed
			break;
			
			// Exit
			case TREQ_EXIT:
			break;
			
			// Unknown
			default:
				SetFont(FNT_PLAIN);
				Print(s:"Got a trader response, but for what action???");
			break;
		}
	}
	
	// ------------------------/
	// Start a trader request
	// ------------------------/
	
	private void M_StartTraderRequest(int id, str ssString, int arg)
	{
		TraderRep.LastArg = arg;
		TraderRep.LastType = id;
		TraderRep.Sending = true;
		TraderRep.SuccessString = ssString;
		
		ACS_NamedTerminate("Rep_TraderWait", 0);
		ACS_NamedExecute("Rep_TraderWait", 0);
		
		NamedRequestScriptPuke("Rep_TraderRequest", id, arg, 0, 0);
	}
	
	// ------------------------/
	// Attempt to buy a weapon
	// ------------------------/
	
	void M_BuyWeapon(int index)
	{
		// See if we have enough money
		if (CheckInventory("Money") < WeaponList[index].Price)
		{
			SetFont(FNT_PLAIN);
			Print(s:"YOU CANNOT AFFORD THAT.");
			return;
		}
		
		// Have it
		if (CheckInventory(WeaponList[index].Class))
		{
			SetFont(FNT_PLAIN);
			Print(s:"YOU HAVE THIS ITEM.");
			return;
		}
		
		// Start a request to buy it
		M_StartTraderRequest(TREQ_BUYWEAPON, StrParam(s:"You bought weapon ", d:index, s:"!"), index);
	}
	
	// ------------------------/
	// Attempt to sell a weapon
	// ------------------------/
	
	void M_SellWeapon(int index)
	{
		// Undroppable
		if (WeaponList[index].Undroppable)
			return;
			
		// Do we have it? We SHOULD, after all
		if (!CheckInventory(WeaponList[index].Class))
		{
			SetFont(FNT_PLAIN);
			Print(s:"YOU DO NOT HAVE THAT ITEM?");
			return;
		}
		
		// Start a request to buy it
		M_StartTraderRequest(TREQ_SELLWEAPON, StrParam(s:"You sold weapon ", d:index, s:"!"), index);
	}
	
	// ------------------------/
	// Attempt to exit the trader menu
	// ------------------------/
	
	void M_ExitTraderMenu()
	{
		// Start a request to buy it
		M_StartTraderRequest(TREQ_EXIT, 0, 0);
	}
}