// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - SPAWNER FUNCTIONALITY
// Contains all code for spawning zombies
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Which animation will zombies use when coming from this spawner?
enum SpawnerAnim
{
	ANIM_NONE,
	ANIM_FLOOR,
};

// All information for a singular spawn spot
struct SpawnerSpot
{
	int Wave;							// FIRST wave this spawner is activated on
	int TID;							// TID of the spawner, if it has one
	int X, Y, Z;						// Position of the spawner in coordinates
	int Builds[SPAWNER_MAXBUILDS];		// Type of zombies that should spawn here
	int BuildsSize;						// Size of array
	
	int Pulses;							// How many pulses has this spawner received?
	
	int Animation;						// Which animation should zombies use?
};

// The spawn "director"
// Contains all of the possible spawn locations and radial spawns
struct SpawnDirect
{
	// For selecting zombies
	int possibles[64];
	int possibleCount;

	// For selecting distance-based spawners
	int radials[32];
	int radialCount;
};

struct SpawnDirect SpawnDirector;

// All possible spawners that have been generated
struct SpawnerSpot spawnSpots[MAXSPAWNERS];
int spawnSpotsSize = 0;

// Zombie classes
#define ZOMBIE_SPAWNERCOUNT 8
#define ZSPAWN_CLASS 0

// Different types of zombies that this spawner supports
#define ZSPAWN_TYPECOUNT 4

str SpawnerTypes[ZOMBIE_SPAWNERCOUNT][1+ZSPAWN_TYPECOUNT] = {

	// Base zombie
	{"ZombieSpawner_Base", "BaseZombie", "BaseZombie", "BaseZombie", "BaseZombie"},
	
	// Facemaw
	{"ZombieSpawner_Vag", "VaginaZombie", "VaginaZombie", "VaginaZombie", "VaginaZombie"},
	
	// Randomized
	{"ZombieSpawner_Random", "BaseZombie", "VaginaZombie", "DogZombie", "SavageImp"},
	
	// Eviscerator
	{"ZombieSpawner_Eviscerator", "Eviscerator", "Eviscerator", "Eviscerator", "Eviscerator"},
	
	// Cranus
	{"ZombieSpawner_Cranus", "Cranus", "Cranus", "Cranus", "Cranus"},
	
	// Dog
	{"ZombieSpawner_Dog", "DogZombie", "DogZombie", "DogZombie", "DogZombie"},
	
	// Blue Dog
	{"ZombieSpawner_BlueDog", "DogZombie", "DogZombie", "DogZombie", "DogZombie"}
};

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Check if we can spawn zombies yet or we are over the pool already
/*
int HasFreeSpots(void)
{
	// How many zombies exist on the map at any one time
	int x = GetZombieCount();
	int x = 1;
	int y;
	
	if (x >= ZombiePool || x >= GetCVar("sv_maxzombies")) 
		y = 0;
	else 
	{
		y = 1; 
		ACS_Execute(66,0,0,0);
	}
	
	// Tell DECORATE if we can spawn or not
	return y;
}
*/

// Get the INDEX of the spawn spot we'd like to use
// This checks distance ONLY, not zombies!

int GetClosestSpawner()
{
	// Reset the director's radial spawner list
	// (Radial spawners are spawners that are close to a player)
	SpawnDirector.radialCount = 0;
	
	// Loop through the entire spawn spot list!
	// This could be optimized when CheckProximity gets added
	
	for (int l=0; l<spawnSpotsSize; l++)
	{
		int isRadial = (spawnSpots[l].Pulses > 0);
			
		// Was a player close enough for this to be a radial spawner?
		// These are prioritized on top of our fallback spawner
		// Radial spawners are stored in a list, and a random is chosen from them
		if (isRadial)
		{
			SpawnDirector.radials[SpawnDirector.radialCount] = l;
			SpawnDirector.radialCount ++;
		}
	}
	
	// We have radial spawners to use, so let's pick from them!
	if (SpawnDirector.radialCount > 0)
	{
		int spawnerChoice = Random(0, SpawnDirector.radialCount-1);
		return SpawnDirector.radials[spawnerChoice];
	}
	
	// Oh dear, we didn't find ANY spawners within range!
	// Let's search very aggressively (and expensively) for one
	
	// (If you see this message, move your spawners closer or add more!)
	
	PrintBold(s:"\cGWARNING: USING AGGRESSIVE DETECTION!\n\cGMOVE YOUR SPAWNERS CLOSER, OR ADD MORE");
	
	return GetClosestSpawner_Aggressive();
}

// Gets the closest spawner in an AGGRESSIVE way!
// This should be used as a fallback ONLY, this is fairly expensive

int GetClosestSpawner_Aggressive()
{
	// Most desirable spawner
	int storedSpawnerID = -1;
	int minSpawnerDist = 99999;
	
	// Reset the director's radial spawner list
	// (Radial spawners are spawners that are close to a player)
	SpawnDirector.radialCount = 0;

	for (int l=0; l<spawnSpotsSize; l++)
	{
		// ONLY if this is within the desired wave
		int spawnerWave = spawnSpots[l].Wave;
		
		// Haven't reached this wave yet!
		// if (waveno < spawnerWave)
			// continue;
			
		// Get coords from the spawner
		int X = spawnSpots[l].X;
		int Y = spawnSpots[l].Y;
		int Z = spawnSpots[l].Z;
		
		// Try to find the closest player
		int minDistance = 99999;
		int isRadial = 0;
		
		// Check each player and find the closest one
		for (int m=0; m<MAXPLAYERS; m++)
		{
			// Does this player exist?
			if (PlayerValid(m))
			{
				// They are closer to us than the last player!
				int dist = fdistancePoint(X, Y, Z, BASE_PLAYER_TID+m);
				if (dist < minDistance)
					minDistance = dist;
					
				// Are they within our spawner proximity?
				// If so, this particular spawner is VALID
				
				if (dist <= SPAWNER_RADIUS)
					isRadial = 1;
			}
		}
		
		// Now we have the closest player to THIS PARTICULAR SPAWNER
		// Is this player closer than the other spawners?
		// If so, we'll use this spawner as a fallback if radial fails
		if (minDistance < minSpawnerDist)
		{
			minSpawnerDist = minDistance;
			storedSpawnerID = l;
		}
		
		// Was a player close enough for this to be a radial spawner?
		// These are prioritized on top of our fallback spawner
		// Radial spawners are stored in a list, and a random is chosen from them
		if (isRadial)
		{
			SpawnDirector.radials[SpawnDirector.radialCount] = l;
			SpawnDirector.radialCount ++;
		}
	}
	
	// We have radial spawners to use, so let's pick from them!
	if (SpawnDirector.radialCount > 0)
	{
		int spawnerChoice = Random(0, SpawnDirector.radialCount-1);
		return SpawnDirector.radials[spawnerChoice];
	}
	
	// No radial spawners, so fallback to whichever one is the closest
	return storedSpawnerID;
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// From a spawner activator, find its index number

int MySpawnerIndex()
{
	int IDX = CheckInventory("SpawnerIndex");
	if (IDX < ZLIST_BASE)
		return -1;
		
	return IDX - ZLIST_BASE;
}

// Find a spawner by its TID
// (Likely not necessary since spawners cache their indexes)

int SpawnerByTID(int tid)
{
	// TODO: WE CAN CHECK TID'S INVENTORY HERE BEFORE LOOPING
	
	for (int l=0; l<spawnSpotsSize; l++)
	{
		if (spawnSpots[l].TID == tid)
			return l;
	}
	
	return -1;
}

// Adds a new zombie spawner to the list!

void AddSpawner()
{
	spawnSpots[spawnSpotsSize].Wave = 1;
	
	// If we actually have a TID, then use that!
	// Otherwise, generate one based on the number
	int AID = ActivatorTID();
	int FinalID = (AID > 0) ? AID : SPAWNER_BASETID + spawnSpotsSize;
	
	Thing_ChangeTID(0, FinalID);
	
	spawnSpots[spawnSpotsSize].TID = FinalID;
	
	spawnSpots[spawnSpotsSize].X = GetActorX(0);
	spawnSpots[spawnSpotsSize].Y = GetActorY(0);
	spawnSpots[spawnSpotsSize].Z = GetActorZ(0);
	
	// Cache spawner index as an inventory item so we can access without looping
	GiveInventory("SpawnerIndex", ZLIST_BASE + spawnSpotsSize);
	
	// Add a build
	spawnSpots[spawnSpotsSize].Builds[0] = BUILD_ALL;
	spawnSpots[spawnSpotsSize].BuildsSize ++;
	
	spawnSpotsSize ++;
}

script "AddSpawner" (void) { AddSpawner(); }

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Find a zombie INDEX for a particular spawner
int ZombieForSpawner(int sIDX)
{
	SpawnDirector.possibleCount = 0;
	
	for (int i=0; i<ZombieListSize; i++)
	{
		// Build 0 is ALL, so we allow all zombies
		if (spawnSpots[sIDX].Builds[0] == BUILD_ALL)
		{
			SpawnDirector.possibles[SpawnDirector.possibleCount] = i;
			SpawnDirector.possibleCount ++;
		}
		
		// Has actual builds
		else
		{
			// LOOP THROUGH THE BUILDS
			for (int b=0; b<spawnSpots[sIDX].BuildsSize; b++)
			{
				// This particular build matches, so skip this build list entirely
				if (ZombieList[i].Build == spawnSpots[sIDX].Builds[b])
				{
					SpawnDirector.possibles[SpawnDirector.possibleCount] = i;
					SpawnDirector.possibleCount ++;
					break;
				}
			}
		}
	}
	
	// None
	if (SpawnDirector.possibleCount == 0)
		return -1;
	
	// Now we have all possible ZOMBIES, so let's pick a random one from them!
	int IDX = Random(0, SpawnDirector.possibleCount - 1);
	return SpawnDirector.possibles[IDX];
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Can we actually spawn a zombie?
// ------------------------/

bool CanSpawnZombie()
{
	if (GameState.State != STATE_WAVE)
		return false;
		
	// No zombies left!
	if (GameState.ZombiePool <= 0)
		return false;
		
	return true;
}

// ------------------------/
// Attempt to actually spawn a zombie!
// ------------------------/

void SpawnZombie()
{
	if (!CanSpawnZombie())
		return;
		
	int SpawnerIndex = GetClosestSpawner();
	
	// Invalid spawner, likely not close enough!
	if (SpawnerIndex == -1)
		return;
	
	int ZombieIndex = ZombieForSpawner(SpawnerIndex);
	
	// Invalid zombie, likely not close enough!
	if (ZombieIndex == -1)
		return;
		
	int X = spawnSpots[SpawnerIndex].X;
	int Y = spawnSpots[SpawnerIndex].Y;
	int Z = spawnSpots[SpawnerIndex].Z;
	
	// A zombie spawned! Decrease our available zombie pool
	if (Spawn(ZombieList[ZombieIndex].Class, X, Y, Z, ZombieList[ZombieIndex].TID))
		GameState.ZombiePool --;
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Get spawner pulse distance, called from DECORATE
// This is just an accessor to our constant

script "GetPulseDistance" (void)
{
	SetResultValue(SPAWNER_RADIUS);
}

// A spawner has received a pulse!

script "SpawnerPulsed" (void)
{
	// Get our spawner ID, if available
	int IDX = MySpawnerIndex();
	if (IDX < 0)
		terminate;
		
	// Do a "heartbeat"
	spawnSpots[IDX].Pulses ++;
	delay(35*SPAWNER_ALIVETIME);
	spawnSpots[IDX].Pulses --;
}

// Spawner pulse script, runs on the server ONLY
// This is called consistently for each player as long as they're alive

script "SpawnerPulseScript" (void)
{
	while (ClassifyActor(0) & ACTOR_ALIVE)
	{
		int X = GetActorX(0);
		int Y = GetActorY(0);
		int Z = GetActorZ(0)+32;
		
		SpawnForced("SpawnerPulseObject",X,Y,Z);
		
		delay(35);
	}
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Handler for "lost" zombies
// ------------------------/

script "SpawnDirectorJanitor"
{
	while (GameState.State == STATE_WAVE)
	{
		delay(35*5);
		CleanupLostZombies();
	}
}

// ------------------------/
// How long should we wait between spawning zombies?
// ------------------------/

int SpawnWaitTime()
{
	return ShaveInt( FixedMul(35.0, Random(SPAWNER_TIMEMIN, SPAWNER_TIMEMAX) ) );
}

// ------------------------/
// Main controller for zombie spawning
// ------------------------/

script "SpawnDirectorAI" (void)
{
	ACS_NamedExecute("SpawnDirectorJanitor",0,0,0,0);
	while (GameState.State == STATE_WAVE)
	{
		delay( SpawnWaitTime() );
		
		// Spawn zombies!
		SpawnZombie();
	}
}

// ------------------------/
// Terminate the Spawn Director
// ------------------------/

void KillSpawnDirector()
{
	ACS_NamedTerminate("SpawnDirectorAI", 0);
	ACS_NamedTerminate("SpawnDirectorJanitor", 0);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Spawn spot to use for boss
// ------------------------/

int GetBossSpawnSpot()
{
	return BOSS_DEF_SPAWNTID;
}

// ------------------------/
// Spawn the final-wave boss!
// ------------------------/

void SpawnBossZombie()
{
	int spotID = GetBossSpawnSpot();
	
	if (SpawnSpotFacing(BOSS_CLASS, spotID, BOSS_TID))
	{
		SetActorState(BOSS_TID, "BossIntro");
		
		// Attempt to spawn a camera for them
		int ang = GetActorAngle(spotID);
		int x = GetActorX(spotID);
		int y = GetActorY(spotID);
		int z = GetActorZ(spotID);
		
		int cam = SpawnCinematicCameraForced(x, y, z, ang);
		if (cam)
			ACS_NamedExecute("BossCinematic", 0, cam, 1);
	}
	else
	{
		SetFont(FNT_PLAIN);
		PrintBold(s:"\cGOH NO, BOSS FAILED TO SPAWN - BAD TID?");
	}
}

// ------------------------/
// Cinematic for the boss
// ------------------------/

script "BossCinematic" (int camTID, int isIntro)
{
	ChangeCamera(camTID, 1, 0);
	
	// Boss intro
	if (isIntro)
	{
		delay(35*2);
		ChangeCamera(0, 0, 0);
		Thing_Destroy(camTID, 1, 0);
	}
	
	// Dying, probably
	else
	{
		delay(35*3);
		
		// STOP looking at the boss camera
		ChangeCamera(0, 0, 0);
		Thing_Destroy(camTID, 1, 0);
		
		int OID = ActivatorTID();
		
		// Freeze all players and make cams for them
		for (int i=0; i<MAXPLAYERS; i++)
		{
			if (!PlayerValid(i))
				continue;
				
			SetActivator(PlayerTID(i));
			
			SetPlayerProperty(0, 1, PROP_TOTALLYFROZEN);
			Thing_Stop(ActivatorTID());
			
			// Make cam
			int X = GetActorX(0);
			int Y = GetActorY(0);
			
			// Get floor Z
			int Z = GetActorFloorZ(ActivatorTID());
			
			SetActorPosition(ActivatorTID(), X, Y, Z, false);
			
			int A = GetActorAngle(0);
			int cam = SpawnCinematicCameraForced(X, Y, Z, A, ActivatorTID());
			
			if (cam)
			{
				SetFont(FNT_PLAIN);
				PrintBold(s:"\cDSetting TP camera with TID ", d:cam, s:" for player ", d:i);
				ChangeCamera(cam, 0, 0);
			}
			else
			{
				SetFont(FNT_PLAIN);
				PrintBold(s:"\cGFAILED TO SET TP CAM FOR PLAYER ", d:i);
			}
		}
		
		SetActivator(OID);
		
		SetFont("BIGFONT");
		PrintBold(s:"YOUR SQUAD SURVIVED");
		
		delay(35*6);
		
		Exit_Normal(0);
	}
}