// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - PERK FUNCTIONALITY
// Contains functionality related to perks
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define PERKS_BASEXP						100
#define PERKS_XP_PER_LEVEL					8.30	// How much to mult XP by per level

// Default per-level mag capacity increase (Added on top of 1.0)
#define DEFAULT_PERK_MAGINC					0.05

// ------------------------/
// Can perks be switched?
// This only applies to the mid-game menu
//
// SERVER
// ------------------------/

bool PerkSwitchAllowed()
{
	if (GameState.State != STATE_TRADER && GameState.State != STATE_SANDBOX)
		return false;
		
	// Just in case
	if (!PlayerState[PlayerNumber()].Alive)
		return false;
		
	return true;
}

// ------------------------/
// Give some experience!
// Requires player as ACTIVATOR!
// ------------------------/

void GiveXP(int xpAmount)
{
	// Not a player! Cannot give XP
	if (!(ClassifyActor(0) & ACTOR_PLAYER))
		return;
		
	// Would giving us the experience level us up?
	int xpMax = GetAmmoCapacity("PerkXP");
	int xpCur = CheckInventory("PerkXP");
	
	// Perform a loop
	// (This could level us up multiple times)
	
	if (xpCur + xpAmount >= xpMax)
	{
		int oldLevel = CheckInventory("PerkLevel");
		
		while (xpAmount > 0)
		{
			xpMax = GetAmmoCapacity("PerkXP");
			xpCur = CheckInventory("PerkXP");
			
			// Would level up?
			if (xpCur + xpAmount >= xpMax)
			{
				// Subtract what's left
				xpAmount -= (xpMax - xpCur);
				GiveInventory("PerkLevel", 1);
				
				// Set our new XP amount
				SetAmmoCapacity("PerkXP", GetXPGoal( CheckInventory("PerkLevel") ));
				
				InvEnsure("PerkXP", 0);
			}
			
			// Wouldn't level us up, just give what's left
			else
			{
				GiveInventory("PerkXP", xpAmount);
				xpAmount = 0;
			}
		}
		
		int newLevel = CheckInventory("PerkLevel");
		
		int OID = ActivatorTID();
		int PN = PlayerNumber();
		
		// This will get sync'd to clients
		PlayerState[PN].PerkLevel = newLevel;
		
		// SetActivator(-1);
		Rep_UpdatePlayerState(PN, -1);
		// SetActivator(OID);
		
		LeveledUp(newLevel - oldLevel);
	}
	
	// Wouldn't level us up, just give it
	else
		GiveInventory("PerkXP", xpAmount);
}

// ------------------------/
// Get a perk XP goal for a specific level
// ------------------------/

int GetXPGoal(int level)
{
	int baseXP = PERKS_BASEXP * 1.0;
	int mult = FixedMul(level * 1.0, PERKS_XP_PER_LEVEL);
	
	return ShaveInt( FixedMul(baseXP, mult) );
}

// ------------------------/
// We leveled up!
// SERVER
// ------------------------/

void LeveledUp(int levels)
{
	SyncPerkBonuses();
	
	SetFont(FNT_PLAIN);
	Print(s:"YOU LEVELED UP");
}

// ------------------------/
// Perk we're currently using
// ------------------------/

int OurPerk()
{
	int PN = IsLocal() ? ConsolePlayerNumber() : PlayerNumber();
	return PlayerState[ PN ].PerkIndex;
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Get the current BUY price for a weapon
// CLIENT + SERVER
// ------------------------/

int GetBuyPrice(int index)
{
	return WeaponList[index].Price;
}

// ------------------------/
// Get the current SELL price for a weapon
// CLIENT + SERVER
// ------------------------/

int GetSellPrice(int index)
{
	int buyPrice = GetBuyPrice(index) * 1.0;
	int dampPrice = FixedMul(buyPrice, WEAPON_SELL_DAMPEN);
	
	return ShaveInt(dampPrice);
}

// ------------------------/
// Get the total mag capacity for a weapon!
// CLIENT + SERVER
// ------------------------/

int GetMagCapacity(int index)
{
	int theMult = 1.0 + (DEFAULT_PERK_MAGINC * CheckInventory("PerkLevel"));
	return ShaveInt( WeaponList[index].PrimaryCap * theMult );
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Change a player's perk
// SERVER
// ------------------------/

void SetPlayerPerk(int plInd, int pkInd)
{
	int AID = ActivatorTID();
	int pTID = PlayerTID(plInd);
	bool PIG = PlayerInGame(plInd);
	
	// Are they in the game?
	
	if (PIG && SetActivator(pTID))
	{
		int oldPI = PlayerState[plInd].PerkIndex;
		
		// Get their current perk data from their ITEMS and cache it
		
		PlayerState[plInd].CachedPerkLevel[oldPI] = CheckInventory("PerkLevel");
		PlayerState[plInd].CachedPerkXP[oldPI] = CheckInventory("PerkXP");
		PlayerState[plInd].CachedPerkXPGoal[oldPI] = GetAmmoCapacity("PerkXP");

		PlayerState[plInd].PerkIndex = pkInd;
		
		ApplyPerkDataFromCache();
		
		SyncPerkBonuses();
	}
	
	else
		PlayerState[plInd].PerkIndex = pkInd;
		
	SetActivator(AID);
}

// ------------------------/
// Sync perk bonuses!
// SERVER
// ------------------------/

void SyncPerkBonuses()
{
	UpdateMagCapacities();
}

// ------------------------/
// Update mag capacities for ALL weapons!
// Done when our perk level changes or we switch perks
//
// (Could this also be done when the weapon
// is selected? We'll see, maybe in the future)
// ------------------------/

void UpdateMagCapacities()
{
	ZTPrint("UpdateMagCapacities()");
	
	for (int i=0; i<WeaponListSize; i++)
	{
		int MC = GetMagCapacity(i);
		SetAmmoCapacity(WeaponList[i].PrimaryClass, MC);
	}
}

// ------------------------/
// Apply perk data from cache if available
// If not, set defaults
// ------------------------/

void ApplyPerkDataFromCache()
{
	int PN = PlayerNumber();
	int PI = PlayerState[PN].PerkIndex;
	
	// If this perk has been cached before, get content from it
		
	if (PlayerState[PN].CachedPerkXPGoal[PI] > 0)
	{
		SetAmmoCapacity("PerkXP", PlayerState[PN].CachedPerkXPGoal[PI]);
		InvEnsure("PerkXP", PlayerState[PN].CachedPerkXP[PI]);
		InvEnsure("PerkLevel", PlayerState[PN].CachedPerkLevel[PI]);
	}
	
	// Not cached? Set some defaults
	
	else
	{
		SetAmmoCapacity("PerkXP", 100);
		InvEnsure("PerkXP", 0);
		InvEnsure("PerkLevel", 0);
	}
}

// ------------------------/
// Generate perk bonus description
// ------------------------/

str MakePerkDescription(int PI, int pLevel)
{
	str desc = PerkList[PI].Description;
	
	return desc;
}