//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Z O M B I E   T O R R E N T   2 
// AMMO RELATED FUNCTIONS
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Struct for a "type" of ammo
// Used for weapons that can have more than one type

struct AmmoType
{
	str Class;
	str Display;
	str Prefix;
	str Icon;
	str BG;
};

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

#define FIRE_MODE_LISTSIZE 				8

// Fire mode enums for HUD DISPLAY ONLY
enum FireModeType
{
	FM_FULL,
	FM_SEMI,
	FM_BURST,
	FM_GRENADE,
	FM_HAMMER,
	FM_SHOTGUN,
	FM_SHOTGUNAUTO,
	FM_CUSTOM
};

str FireModeImages[FIRE_MODE_LISTSIZE] = {
	"KFFMAUTO",
	"KFFMSING",
	"KFFMBURS",
	"KFFMGREN",
	"KFFMMELE",
	"KFFMSHSA",
	"KFFMSHFA",
	"KFFMAUTO"
};

// ------------------------/
// Which image should we show on the HUD?
// ------------------------/

str GetFireModeImage()
{
	int WI = LocalWeaponIndex();
	if (WI < 0)
		return "";
	
	return FireModeImages[ WeaponList[WI].DefaultFireMode ];
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Can this weapon be reloaded?
// ------------------------/

bool ReloadAllowed()
{
	int WI = LocalWeaponIndex();
	
	if (CheckInventory( WeaponList[WI].SecondaryClass ) <= 0)
		return false;
	
	int ammoP = CheckInventory( WeaponList[WI].PrimaryClass );
	int ammoMax = GetMagCapacity(WI);
	
	return (ammoP < ammoMax);
}

script "ReloadAllowed" (void) { SetResultValue( ReloadAllowed() ); }

// ------------------------/
// Perform ammo-based reload magic
// ------------------------/

void ReloadMagic()
{
	int WI = LocalWeaponIndex();
	
	str ClassP = WeaponList[WI].PrimaryClass;
	str ClassS = WeaponList[WI].SecondaryClass;
	int CapP = GetMagCapacity(WI);
	
	// How much ammo can we possibly take?
	int CanTake = min( CapP - CheckInventory(ClassP), CheckInventory(ClassS) );
	if (CanTake <= 0)
		return;
		
	TakeInventory(ClassS, CanTake);
	GiveInventory(ClassP, CanTake);
}

script "ReloadMagic" (void) { ReloadMagic(); }

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Get the WeaponList index for our current grenade type
// ------------------------/

int GrenadeIndex()
{
	str gC = PerkList[ OurPerk() ].GrenadeClass;
	return WeaponByType(gC);
}

// ------------------------/
// How many grenades do we currently have?
// ------------------------/

int GrenadesLeft()
{
	int GI = GrenadeIndex();
	
	if (GI < 0)
		return 0;
		
	return CheckInventory(WeaponList[GI].PrimaryClass);
}

script "CanTossGrenade"
{
	if ( GrenadesLeft() > 0 )
		SetResultValue(1);
	else
		SetResultValue(0);
}

// ------------------------/
// Consume an actual grenade
// ------------------------/

script "ConsumeGrenade"
{
	int GI = GrenadeIndex();
	if (GI > 0)
		TakeInventory( WeaponList[GI].PrimaryClass, 1 );
}