// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - REPLICATION
// Mostly RELIABLE things from client -> server
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Flag spacing for perk level
#define PERKLEVEL_FLAGSPACING				5000

enum RepTypes
{
	MSG_ZOMBIECOUNT,				// Zombie count
	MSG_TIMECOUNT,					// Countdown / trader time
	MSG_GAMESTATE,					// The game state
	MSG_PLAYERSTATE,				// A player's state
	MSG_CURTRADER,					// Set the current trader ID
};

enum PlayerStateTypes
{
	PS_ALIVE,						// Just make us alive
	PS_DEAD,						// We're dead
	PS_READY,						// We're ready
	PS_NOTREADY,					// We're not ready yet
	PS_RESET,						// Not alive, not ready
	PS_ALIVEANDREADY,				// Alive and ready
};

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

struct ZT2Replication
{
	bool RequestingState;					// Requesting the initial gamestate!
	bool SendingResolution;					// Telling the server what our res is
	
	bool SendingPerkChange;					// We're trying to change our perk!
};

struct ZT2Replication Replication;

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// First segment of a player's state
//
// >=10000 = alive
// >=5000 = ready
// Remaining is player number
// ------------------------/

int ToStateA(int ind)
{
	int num = 0;
	
	if (PlayerState[ind].Alive)
		num += 10000;
	if (PlayerState[ind].Ready)
		num += 5000;
		
	num += ind;
	
	return num;
}

// ------------------------/
// Second segment of a player's state
// Just perk hash (a bit complicated, but whatever)
//
// (CALL THIS ON THE SERVER)
// ------------------------/

int ToStateB(int ind)
{
	int sB = PerkList[PlayerState[ind].PerkIndex].Hash;
	
	// Ideally, hashes should only go up to about 1500 or so
	// As a result, let's use 5000 per perk level
	
	// Let's not rely on in-game players
	/*
	if (PlayerState[ind].Alive && PlayerInGame(ind))
	{
		int pTID = PlayerTID(ind);
		sB += (PERKLEVEL_FLAGSPACING * CheckActorInventory(pTID, "PerkLevel"));
	}
	*/
	
	sB += PERKLEVEL_FLAGSPACING * PlayerState[ind].PerkLevel;
	
	return sB;
}

// ------------------------/
// Read A and B states of a player!
// CLIENT
// ------------------------/

void ReadPlayerState(int numA, int numB)
{
	bool A, R;
	int ply;
	
	if (numA >= 10000)
	{
		numA -= 10000;
		A = true;
	}
	
	if (numA >= 5000)
	{
		numA -= 5000;
		R = true;
	}
	
	ply = numA;
	
	// Find the appropriate perk data
	int pLevel = 0;
	int pHash = numB;
	
	// Divide by 5000
	int div = numB / PERKLEVEL_FLAGSPACING;
	
	if (div >= 1)
	{
		pLevel = div;
		pHash = numB - (PERKLEVEL_FLAGSPACING * pLevel);
	}
	else
		pHash = numB;

	int pIndex = PerkByHash(pHash);
	
	if (pIndex >= 0)
		PlayerState[ply].PerkIndex = pIndex;
	else
		PlayerState[ply].PerkIndex = 0;
		
	PlayerState[ply].PerkLevel = pLevel;
	
	ZTPrint(StrParam(n: ply+1, s:" - Alive: ", d:A, s:", Ready: ", d:R, s:", Num: ", d:ply, s:", Perk: ", d:pHash, s:", Perk Level: ", d:pLevel));
	
	PlayerState[ply].Alive = A;
	PlayerState[ply].Ready = R;
	
	// Wait a minute, is this us?
	// If we were sending our perk data, stop sending it
	//		(Could this cause a race condition? Not sure)
	
	if (ply == ConsolePlayerNumber() && Replication.SendingPerkChange)
		Replication.SendingPerkChange = false;
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// ResolutionSender
// Client [Wait]
// ------------------------/

script "ResolutionSender" (int ResX, int ResY)
{
	Replication.SendingResolution = true;
	
	while (true)
	{
		NamedRequestScriptPuke("ResolutionPing", ResX, ResY, ConsolePlayerNumber(), 0);
		delay(REPLICATION_INTERVAL);
		
		// Received a response
		if (!Replication.SendingResolution)
			break;
	}
}

// ------------------------/
// ResolutionPing
// Client -> Server
// ------------------------/

script "ResolutionPing" (int ResX, int ResY) NET
{
	int PN = PlayerNumber();
	
	PlayerState[PN].ResX = ResX;
	PlayerState[PN].ResY = ResY;
	ACS_NamedExecuteAlways("ResolutionPong", 0);
}

// ------------------------/
// ResolutionPong
// Server -> Client
// ------------------------/

script "ResolutionPong" CLIENTSIDE
{
	if( WrongClient() )
		terminate;
		
	Replication.SendingResolution = false;
	
	ZTPrint("ResolutionPong");
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// StateRequest
// Client [Wait]
// ------------------------/

script "StateRequest_Wait"
{
	Replication.RequestingState = true;
	
	while (true)
	{
		NamedRequestScriptPuke("StateRequest_Ping", 0, 0, 0, 0);
		delay(REPLICATION_INTERVAL);
		
		// Received a response
		if (!Replication.RequestingState)
			break;
	}
};

// ------------------------/
// StateRequest
// Client -> Server
// ------------------------/

script "StateRequest_Ping" NET
{
	// Send a pong response back to the originator
	ACS_NamedExecute("StateRequest_Pong", 0);
	
	// Now let's actually send updated gamestates to people who don't have them
	// (Doing a check in the client script is useless, since
	// everyone will execute it regardless)
	
	// Timing / zombie count
	Rep_UpdateCount();
	
	// All player information
	for (int i=0; i<MAXPLAYERS; i++)
	{
		if (!PlayerValid(i))
			continue;
				
		Rep_UpdatePlayerState(i, -1);
	}
}

// ------------------------/
// StateRequest
// Server -> Client
// ------------------------/

script "StateRequest_Pong" CLIENTSIDE
{
	if (WrongClient())
		terminate;
	
	Replication.RequestingState = false;
	
	ZTPrint("StateRequest_Pong");
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Send a gamestate update message
// Server -> Client
// ------------------------/

script "UpdateGameState" (int msg, int a, int b) CLIENTSIDE
{
	switch (msg)
	{
		// Zombies left
		case MSG_ZOMBIECOUNT:
			GameState.HUD.ZombiesLeft = a;
			ZTPrint("MSG_ZOMBIECOUNT");
			
			// Ensure we're in a wave (Is this a good idea?)
			if (IsClient())
				GameState.State = STATE_WAVE;
		break;
		
		// Timing variable
		case MSG_TIMECOUNT:
			GameState.HUD.TimeLeft = a;
			
			if (IsClient())
				GameState.WaveTimer = a;
				
			ZTPrint("MSG_TIMECOUNT");
		break;
		
		// Game state
		case MSG_GAMESTATE:
			GameState.State = a;
			GameState.Wave = b;
			ReceivedStateChange();
			ZTPrint("MSG_GAMESTATE");
		break;
		
		// Player state
		case MSG_PLAYERSTATE:
			ReadPlayerState(a, b);
		break;
		
		// Current trader number is upgraded
		case MSG_CURTRADER:
			GameState.CurrentTrader = a;
		break;
	}
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Send gamestate to clients
// ------------------------/

void Rep_UpdateState()
{
	ACS_NamedExecuteAlways("UpdateGameState", 0, MSG_GAMESTATE, GameState.State, GameState.Wave);
}

// ------------------------/
// Sends time, etc. to clients
// ------------------------/

void Rep_UpdateCount()
{
	// Lobby has no time-related things
	if (GameState.State == STATE_LOBBY)
		return;
		
	// Send a timing value
	if (GameState.State == STATE_TRADER || GameState.State == STATE_COUNTDOWN)
		ACS_NamedExecuteAlways("UpdateGameState", 0, MSG_TIMECOUNT, GameState.WaveTimer);
	else
		ACS_NamedExecuteAlways("UpdateGameState", 0, MSG_ZOMBIECOUNT, RemainingZombies(), 600);
}

// ------------------------/
// Send trader ID to clients
// ------------------------/

void Rep_UpdateTraderID()
{
	ACS_NamedExecuteAlways("UpdateGameState", 0, MSG_CURTRADER, GameState.CurrentTrader);
}

// ------------------------/
// The server sent us a state change
// ------------------------/
void ReceivedStateChange()
{
	PrintBold(s:"PN: ", d:PlayerNumber());
	
	// Wave REALLY started!
	if (GameState.State == STATE_WAVE)
		ShowFightImage();
		
	// Trader time!
	// Doing AmbientSound should be fine, on the client end
	else if (GameState.State == STATE_TRADER)
		AmbientSound("wave/completed", 127);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Send a player state message
//
// This should ONLY be used for messages that 
// the client needs knowledge of
//
// SERVER
// ------------------------/

void Rep_UpdatePlayerState(int p, int s = -1)
{
	// Only if we specified a state
	if (s >= 0)
		HandlePlayerState(p, s);
	
	int PSA = ToStateA(p);
	int PSB = ToStateB(p);
	
	ACS_NamedExecuteAlways("UpdateGameState", 0, MSG_PLAYERSTATE, PSA, PSB);
}
	
// ------------------------/
// Actually HANDLE a state change
// SERVER
// ------------------------/
void HandlePlayerState(int player, int state)
{
	int CPN = ConsolePlayerNumber();
	bool isLocalPlayer = (IsLocal() && player == CPN);
	
	GameState.HUD.SquadSyncPending = true;
	
	// Which state?
	switch (state)
	{
		// Alive!
		case PS_ALIVE:
			PlayerState[player].Alive = true;
			break;
			
		// Dead!
		case PS_DEAD:
			PlayerState[player].Alive = false;
			break;
			
		// Ready!
		case PS_READY:
			PlayerState[player].Ready = true;
			break;
			
		// Not ready
		case PS_NOTREADY:
			PlayerState[player].Ready = false;
			break;
			
		// Alive AND ready
		case PS_ALIVEANDREADY:
			PlayerState[player].Alive = true;
			PlayerState[player].Ready = true;
			break;
			
		// Not alive, not ready
		case PS_RESET:
			PlayerState[player].Ready = false;
			PlayerState[player].Alive = false;
			PlayerState[player].InTrader = false;
			break;
	}
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Types of requests (or responses) we can send during tradertime
enum TraderRequestType
{
	// Requests
	TREQ_BUYWEAPON,			// Buy a weapon
	TREQ_SELLWEAPON,		// Sell a weapon
	TREQ_BUYUPGRADE,		// Buy an upgrade
	TREQ_SELLUPGRADE,		// Sell an upgrade
	TREQ_BUYAMMO,			// Refill ammo
	TREQ_BUYARMOR,			// Refill armor
	TREQ_EXIT,				// Exit the trader
	
	// Responses
	TRES_SUCCESS,			// Success!
	TRES_FAILED,			// This request failed
	TRES_BROKE,				// Not enough money
	TRES_HAS,				// We already have this item or are capped
	TRES_NOTHAVE,			// We don't have the item ???
};

// ------------------------/
// Actual trader request script
// Client -> Server
// ------------------------/

script "Rep_TraderRequest" (int type, int arg) NET
{
	int result = TRES_SUCCESS;
	
	switch (type)
	{
		//-- SELL A WEAPON
		case TREQ_SELLWEAPON:
			result = TF_SellWeapon(arg);
		break;
		
		//-- BUY A WEAPON
		case TREQ_BUYWEAPON:
			result = TF_BuyWeapon(arg);
		break;
		
		//-- EXIT TRADER
		case TREQ_EXIT:
			result = TF_ExitMenu();
		break;
	}
	
	// Send it back
	ACS_NamedExecuteAlways("Rep_TraderResponse", 0, result);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Attempt a perk change!
// Called from CLIENT
// ------------------------/

void AttemptPerkChange(int newPerk, bool fromLobby = true)
{
	Replication.SendingPerkChange = true;
	NamedRequestScriptPuke("PerkChange_Request", ConsolePlayerNumber(), newPerk, fromLobby ? 1 : 0);
	ACS_NamedExecuteAlways("PerkChange_Timer", 0);
}

// ------------------------/
// Perk change TIMEOUT
// CLIENT ONLY
//
// This times out if we don't receive a response in time
// ------------------------/
script "PerkChange_Timer"
{
	delay(REPLICATION_INTERVAL);
	Replication.SendingPerkChange = false;
}

// ------------------------/
// Perk change REQUEST
// CLIENT -> SERVER
// ------------------------/

script "PerkChange_Request" (int CPN, int newPerk, int fromLobby) NET
{
	// Only set index if we're allowed to switch
	if (fromLobby || (!fromLobby && PerkSwitchAllowed()))
		SetPlayerPerk(CPN, newPerk);
		
	// Send this player's NEW data to all clients
	// When the sending player receives it, they'll act accordingly
	
	Rep_UpdatePlayerState(CPN, -1);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Play a music track!
// SERVER -> CLIENT
// ------------------------/

script "PlayMusicTrack_Rep" (int trackType, int trackHash) CLIENTSIDE
{
	ZTPrint("REP");
	ParsePlaylistRequest(trackType, trackHash);
}